module Data.Decimal.Print where

-- TODO: add engineering
-- add user defineable printing specs, e.g. %2.2f

import Data.Decimal.Representation

instance Show Decimal where
    show = showScientific

showScientific :: Decimal -> String
showScientific (QuietNaN  Positive d) = "NaN"   ++ (maybe "" show d)
showScientific (QuietNaN  Negative d) = "-NaN"  ++ (maybe "" show d)
showScientific (SignalNaN Positive d) = "sNaN"  ++ (maybe "" show d)
showScientific (SignalNaN Negative d) = "-sNaN" ++ (maybe "" show d)
showScientific (Infinity Positive)    = "Infinity"
showScientific (Infinity Negative)    = "-Infinity"
showScientific (Decimal sign coef exp) =
    let coefStr = show coef
	signStr = case sign of 
		       Positive -> "" 
		       Negative -> "-"
	adjExp  = exp + (fromIntegral (length coefStr)) - 1
	in
	signStr ++ (showDec coefStr exp adjExp)
    where showDec :: String -> Integer -> Integer -> String
	  showDec coefStr exp adjExp
	      | (exp == 0) && (adjExp >= (-6)) =
		  coefStr
	      | (exp < 0) && (adjExp >= (-6)) = 
		  if (length coefStr) <= (fromIntegral (abs exp))
		     then let str = (reverse coefStr) ++ "000000"
		              (before:after) = 
				  reverse (take (fromIntegral (abs exp) + 1) str )
		          in
			  (before:'.':after)
		     else let (retfa,erofeb) = 
				  splitAt (fromIntegral (abs exp)) (reverse coefStr)
                          in
			  (reverse erofeb ++ "." ++ reverse retfa)
	      | otherwise =
		  let coefStr' =
			case coefStr of
		             [c] -> coefStr
	                     (c:cs) -> (c:'.':cs)
		      expStr =
			  if adjExp > 0
			     then ('+' : show adjExp)
			     else (show adjExp)
		      in
		      coefStr' ++ "E" ++ expStr

testValuesScientific =
    [ ((Decimal Positive 123 0),"123")
    , ((Decimal Negative 123 0), "-123")
    , ((Decimal Positive 123 1), "1.23E+3")
    , ((Decimal Positive 123 3), "1.23E+5")
    , ((Decimal Positive 123 (-1)), "12.3")
    , ((Decimal Positive 123 (-5)), "0.00123")
    , ((Decimal Positive 123 (-10)), "1.23E-8")
    , ((Decimal Negative 123 (-12)), "-1.23E-10")
    , ((Decimal Positive 0 0), "0")
    , ((Decimal Positive 0 (-2)), "0.00")
    , ((Decimal Positive 0 2), "0E+2")
    , ((Decimal Negative 0 0), "-0")
    , ((Decimal Positive 5 (-6)), "0.000005")
    , ((Decimal Positive 50 (-7)), "0.0000050")
    , ((Decimal Positive 5 (-7)), "5E-7")
    , ((Infinity Positive), "Infinity")
    , ((Infinity Negative), "-Infinity")
    , ((QuietNaN Positive Nothing), "NaN")
    , ((QuietNaN Positive (Just 123)), "NaN123")
    , ((SignalNaN Negative Nothing), "-sNaN")
    ]

checkValueScientific (d,s) = (showScientific d) == s

