module Data.Decimal.ReadP where

import Data.Char
import Data.Decimal.Representation
import Text.ParserCombinators.ReadP

type Digits = [Char]

instance Read Decimal where
    readsPrec n = readP_to_S decimal

decimal :: ReadP Decimal
decimal =
    do s <- sign
       do (nan s)
	  +++
	  (infinity s)
	  +++
	  do (beforePoint,afterPoint) <- decimalPart
	     ep <- (do indicator
		       exponentPart
		    <++
		    return 0)
	     let exp = ep - (fromIntegral (length afterPoint))
		 coef = read (beforePoint ++ afterPoint)
             return (Decimal s coef exp)

decimalPart :: ReadP (Digits, Digits)
decimalPart =
    do char '.'
       d2 <- digits
       return ("",d2)
    +++
    do d1 <- digits
       d2 <- (char '.' >> (digits <++ return ""))
	     <++ return ""
       return (d1,d2)

exponentPart :: ReadP Integer
exponentPart =
    do s <- sign
       d <- digits
       case s of
	    Positive -> return (read d)
	    Negative -> return (-(read d))

digits :: ReadP Digits
digits = 
    munch1 isDigit

sign :: ReadP Sign
sign =
    do char '+'
       return Positive
    <++
    do char '-'
       return Negative
    <++
    do return Positive

indicator :: ReadP Char
indicator =
    char 'E' +++ char 'e'

infinity :: Sign -> ReadP Decimal
infinity s =
    do istring "Inf"
       istring "inity" <++ return ""
       return (Infinity s)

nan :: Sign -> ReadP Decimal
nan s =
    do istring "NaN"
       d <- maybep digits
       return (QuietNaN s (maybe Nothing read d))
    +++
    do istring "sNaN"       
       d <- maybep digits
       return (SignalNaN s (maybe Nothing read d))

maybep :: ReadP a -> ReadP (Maybe a)
maybep p = 
    (p >>= (return . Just))
    <++
    (return Nothing)


istring :: String -> ReadP String
istring str =
    mapM choice (map (\c -> [char (toUpper c), char (toLower c)]) str)


