{-# LANGUAGE DeriveDataTypeable #-}
module Data.Decimal.Representation where

import Data.Word
import Data.Generics(Typeable(..),Data(..))

data Precision 
    = Single
    | Double
    | Precision Word -- ^ the maximum number of significant digits 
                     -- that can result from an arithmetic operation
      deriving (Show, Typeable, Data)

data Rounding
    = RoundDown
    | RoundHalfUp
    | RoundHalfEven
    | RoundCeiling
    | RoundFloor
    | RoundHalfDown
    | RoundUp
      deriving (Show, Typeable, Data)

data Signals
    = Signals { clamped          :: Bool
	      , divisionByZero   :: Bool
	      , inexact          :: Bool
	      , invalidOperation :: Bool
	      , overflow         :: Bool
	      , rounded          :: Bool
	      , subNormal        :: Bool
	      , underFlow        :: Bool
	      }
      deriving (Show, Typeable, Data)
{-
    = Clamped
    | DivisionByZero
    | Inexact
    | InvalidOperation
    | Overflow
    | Rounded
    | SubNormal
    | Underflow
      deriving Show
-}
data Context 
    = Context { flags        :: Signals
	      , trapEnablers :: Signals
	      , precision    :: Precision
	      , rounding     :: Rounding
	      }
      deriving (Show, Typeable, Data)

basicContext =
    Context { flags = 
	      Signals { clamped          = False
		      , divisionByZero   = False
		      , inexact          = False
		      , invalidOperation = False
		      , overflow         = False
		      , rounded          = False
		      , subNormal        = False
		      , underFlow        = False
		      }
	   , trapEnablers = 
	      Signals { clamped          = True
		      , divisionByZero   = True
		      , inexact          = False
		      , invalidOperation = True
		      , overflow         = True
		      , rounded          = False
		      , subNormal        = False
		      , underFlow        = True
		      }
	   , precision = Precision 9
	   , rounding  = RoundHalfUp
	   }


extendedContext =
    Context { flags = 
	      Signals { clamped          = False
		      , divisionByZero   = False
		      , inexact          = False
		      , invalidOperation = False
		      , overflow         = False
		      , rounded          = False
		      , subNormal        = False
		      , underFlow        = False
		     }
	   , trapEnablers = 
	      Signals { clamped          = False
		      , divisionByZero   = False
		      , inexact          = False
		      , invalidOperation = False
		      , overflow         = False
		      , rounded          = False
		      , subNormal        = False
		      , underFlow        = False
		      }
	   , precision = Single
	   , rounding  = RoundHalfEven
	   }

data Sign 
    = Positive
    | Negative
      deriving (Show, Eq, Typeable, Data)

data Decimal 
    = Decimal Sign Integer Integer -- ^ FIXME
    | Infinity Sign
    | QuietNaN Sign (Maybe Integer)
    | SignalNaN Sign (Maybe Integer)
      deriving (Typeable, Data)

