module Data.Decimal.Test where

import Test.HUnit

import Data.Decimal.Representation
import Data.Decimal.Print
import Data.Decimal.ReadP
import Data.Decimal.Operations

testAbs =
    [ TestCase (assertEqual  "abs 2.1 = 2.1" "2.1" (show $ abs ((read "2.1") :: Decimal)))
    , TestCase (assertEqual  "abs -100 = 100" "100" (show $ abs ((read "-100") :: Decimal)))
    , TestCase (assertEqual  "abs 101.5 = 101.5" "101.5" (show $ abs ((read "101.5") :: Decimal)))
    , TestCase (assertEqual  "abs -101.5 = 101.5" "101.5" (show $ abs ((read "-101.5") :: Decimal)))
    ]

testAdd =
    [ TestCase (assertEqual  "12 + 7.00 = 19.00" "19.00" (show $ ((read "12") :: Decimal) + (read "7.00")))
    , TestCase (assertEqual  "1E+2 + 1E+4 = 1.01E+4" "1.01E+4" (show $ (read "1E+2" :: Decimal) + (read "1E+4")))
    , TestCase (assertEqual  "1.3 - 1.07 = 0.23" "0.23" (show $ ((read "1.3") :: Decimal) - (read "1.07")))
    , TestCase (assertEqual  "1.3 - 1.30 = 0.00" "0.00" (show $ ((read "1.3") :: Decimal) - (read "1.30")))
    , TestCase (assertEqual  "1.3 - 2.07 = -0.77" "-0.77" (show $ ((read "1.3") :: Decimal) - (read "2.07")))
    ]

testCompare =
    [ TestCase (assertEqual "compare 2.1 3 = LT" LT (compare ((read "2.1") :: Decimal) (read "3")))
    , TestCase (assertEqual "compare 2.1 2.1 = EQ" EQ (compare ((read "2.1") :: Decimal) (read "2.1")))
    , TestCase (assertEqual "compare 2.1 2.10 = EQ" EQ (compare ((read "2.1") :: Decimal) (read "2.10")))
    , TestCase (assertEqual "compare 3 2.1 = GT" GT (compare ((read "3") :: Decimal) (read "2.1")))
    , TestCase (assertEqual "compare 2.1 -3 = GT" GT (compare ((read "2.1") :: Decimal) (read "-3")))
    , TestCase (assertEqual "compare -3 2.1 = LT" LT (compare ((read "-3") :: Decimal) (read "2.1")))
    ]

testDivide =
    [ TestCase (assertEqual "1 / 3 = 0.333333333" "0.333333333" (show $ ((read "1" :: Decimal) / (read "3"))))
    , TestCase (assertEqual "2 / 3 = 0.666666667" "0.666666667" (show $ ((read "2" :: Decimal) / (read "3"))))
    , TestCase (assertEqual "5 / 2 = 2.5" "2.5" (show $ ((read "5" :: Decimal) / (read "2"))))
    , TestCase (assertEqual "1 / 10 = 0.1" "0.1" (show $ ((read "1" :: Decimal) / (read "10"))))
    , TestCase (assertEqual "12 / 12 = 1" "1" (show $ ((read "12" :: Decimal) / (read "12"))))
    , TestCase (assertEqual "8.00 / 2 = 4.00" "4.00" (show $ ((read "8.00" :: Decimal) / (read "2"))))
    , TestCase (assertEqual "2.400 / 2.0 = 1.20" "1.20" (show $ ((read "2.400" :: Decimal) / (read "2.0"))))
    , TestCase (assertEqual "1000 / 100 = 10" "10" (show $ ((read "1000" :: Decimal) / (read "100"))))
    , TestCase (assertEqual "1000 / 1 = 1000" "1000" (show $ ((read "1000" :: Decimal) / (read "1"))))
    , TestCase (assertEqual "2.40E+6 / 2 = 1.20E+6" "1.20E+6" (show $ ((read "2.40E+6" :: Decimal) / (read "2"))))
    ]

testMultiply =
    [ TestCase (assertEqual "1.20 * 3 = 3.60" "3.60" (show $ ((read "1.20") :: Decimal) * (read "3" :: Decimal)))
    , TestCase (assertEqual "7 * 3 = 21" "21" (show $ ((read "7") :: Decimal) * (read "3" :: Decimal)))
    , TestCase (assertEqual "0.9 * 0.8 = 0.72" "0.72" (show $ ((read "0.9") :: Decimal) * (read "0.8" :: Decimal)))
    , TestCase (assertEqual "0.9 * -0 = -0.0" "-0.0" (show $ ((read "0.9") :: Decimal) * (read "-0" :: Decimal)))
    , TestCase (assertEqual "654321 * 654321 = 4.28135971E+11" "4.28135971E+11" (show $ ((read "654321") :: Decimal) * (read "654321" :: Decimal)))
    ]

testSpecial =
    [ TestCase (assertEqual "Infinity + 1 = Infinity" "Infinity" (show $ (read "Infinity" :: Decimal) + (read "1"))) 
    , TestCase (assertEqual "NaN + 1 = NaN" "NaN" (show $ (read "NaN" :: Decimal) + (read "1"))) 
    , TestCase (assertEqual "NaN + Infinity = NaN" "NaN" (show $ (read "NaN" :: Decimal) + (read "Infinity"))) 
    , TestCase (assertEqual "1 - Infinity = -Infinity" "-Infinity" (show $ (read "1" :: Decimal) - (read "Infinity"))) 
    , TestCase (assertEqual "-1 * Infinity = -Infinity" "-Infinity" (show $ (read "-1" :: Decimal) * (read "Infinity"))) 
    , TestCase (assertEqual "-0 - 0  = -0" "-0" (show $ (read "-0" :: Decimal) - (read "0"))) 
    , TestCase (assertEqual "-1 * 0  = -0" "-0" (show $ (read "-1" :: Decimal) * (read "0"))) 
    , TestCase (assertEqual "1 / 0  = Infinity" "Infinity" (show $ (read "1" :: Decimal) / (read "0"))) 
    , TestCase (assertEqual "1 / -0  = -Infinity" "-Infinity" (show $ (read "1" :: Decimal) / (read "-0"))) 
    , TestCase (assertEqual "-1 / 0  = -Infinity" "-Infinity" (show $ (read "-1" :: Decimal) / (read "0"))) 
    ]

tests = runTestTT $ test (testAbs ++ testAdd ++ testCompare ++ testMultiply ++ testDivide {- ++ testSpecial -})